import subprocess
import json
import requests
import sys
from requests.auth import HTTPBasicAuth
from urllib.parse import urlparse
import xml.etree.ElementTree as et


def requestSettings(section, oldPlugin = False):
    """
    Запросить настройки через утилиту currentsettings
    :param section: название секции ini настроек или название старого плагина
    :param oldPlugin: запросить настройки для старого плагина
    :return: справочник с настройками
    """
    process = subprocess.Popen(["sudo", "/linuxcash/cash/bin/currentsettings", "-p" if oldPlugin else "-s", section],
                               stdout=subprocess.PIPE)
    process.wait()
    data, err = process.communicate()
    return json.loads(data)


def checkLogical(value):
    """
    Проверить логическое значение из настроек
    """
    return value in ["ok", "yes", "enable", "1", "true", "allow", "on", "y", "да"]


def getValue(settings, key):
    """
    Получить занчение настройки
    :param settings: справочник с настройками
    :param key: ключ
    :return: None если ключ отсутствует в справочнике, иначе - значение настройки
    """
    lowCaseDict = dict((k.lower(), v) for k, v in settings.items())
    if key.lower() not in lowCaseDict.keys():
        return None
    return lowCaseDict[key.lower()]


def pluginEnabled(pluginName):
    """
    Проверить включен ли плагин
    :param pluginName: название плагина
    :return: true - плагин включен, false - выключен
    """
    settings = requestSettings("plugins")
    return checkLogical(getValue(settings, pluginName))


def parseArtixBonusSettings(paresedXml):
    """
    Привести распаршенную XML-ку к более удобному виду
    :param paresedXml: XML преобразованный в JSON
    :return: справочник с настройками
    """
    if not paresedXml:
        return {}
    result = {}
    for prop in paresedXml["objects"]["object"]["property"]:
        if prop["name"] == "address":
            result["host"] = prop["map"]["entry"]["key"]
            result["port"] = prop["map"]["entry"]["value"]
        else:
            result[prop["name"]] = (prop["value"] if prop["value"] else "")
    return result


def testArtixBonus(pluginName):
    """
    Проверка соединения с бонусными сервисами для старых плагинов Artix
    :param pluginName: имя плагина (artixbonus или artixcertificate)
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    settings = parseArtixBonusSettings(requestSettings(pluginName, True))
    if not settings:
        return 2
    host = getValue(settings, "host")
    port = getValue(settings, "port")
    if not host or not port:
        raise Exception("Не указан адрес сервера или порт")
    address = host + ":" + port
    if address[:4] != "http":
        address = "http://" + address
    timeout = getValue(settings, "connectTimeout")
    timeout = 10 if not timeout else int(timeout) / 1000
    try:
        resp = requests.get(address + "/ACC/rest/v1/version", timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testArtixCoupon():
    """
    Проверка соединения с сервисом купонов Artix
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled("OnlineCoupon"):
        return 2
    settings = requestSettings("OnlineCoupon")
    host = getValue(settings, "host")
    port = getValue(settings, "port")
    if not host or not port:
        raise Exception("Не указан адрес сервера или порт")
    address = host + ":" + port
    if address[:4] != "http":
        address = "http://" + address
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(address + "/ACC/rest/v1/version", timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testOnlineCards():
    """
    Проверка соединения с сервисом онлайн карт
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    settings = requestSettings("ArtixBonus.OnlineCards")
    if not checkLogical(getValue(settings, "enable")):
        return 2
    rawUrl = getValue(settings, "url")
    if not rawUrl:
        raise Exception("Не указан адрес сервера")
    url = urlparse(rawUrl)
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(url._replace(path="/version").geturl(), timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testDiscountCounters():
    """
    Проверка соединения с сервисом счетчиков
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    settings = requestSettings("DiscountCounters")
    if not checkLogical(getValue(settings, "enable")):
        return 2
    host = getValue(settings, "host")
    port = getValue(settings, "port")
    if not host or not port:
        raise Exception("Не указан адрес сервера или порт")
    address = host + ":" + port
    if address[:4] != "http":
        address = "http://" + address
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(address + "/version", timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testCardIssuance():
    """
    Проверка соединения с сервисом выдачи карт
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled("CardIssuance"):
        return 2
    settings = requestSettings("CardIssuance")
    rawUrl = getValue(settings, "url")
    if not rawUrl:
        raise Exception("Не указан адрес сервера")
    url = urlparse(rawUrl)
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    try:
        resp = requests.get(url._replace(path="/version").geturl(), timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testRsLoyalty(pluginName):
    """
    Проверка соединения с сервером лояльности RS Loyalty
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled(pluginName):
        return 2
    settings = requestSettings("RSLoyalty")
    rawHosts = getValue(settings, "hosts")
    if not rawHosts:
        raise Exception("Не указан адрес сервера")
    hosts = str(rawHosts).split(',')
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    headers = {
        "SOAPAction": "http://tempuri.org/IRSLoyaltyService/Ping",
        "Content-Type": "text/xml; charset=UTF-8",
    }
    auth = HTTPBasicAuth(getValue(settings, "user"), getValue(settings, "password"))
    data = et.Element("soapenv:Envelope")
    data.set("xmlns:soapenv", "http://schemas.xmlsoap.org/soap/envelope/")
    data.set("xmlns:tem", "http://tempuri.org/")
    body = et.SubElement(data, "soapenv:Body")
    et.SubElement(body, "tem:Ping")
    for url in hosts:
        try:
            resp = requests.post(url, data=et.tostring(data), headers=headers, timeout=timeout, auth=auth)
            resp.close()
            if resp.status_code == 200:
                return 0
        except Exception:
            return 1
    return 1


def testKilBil():
    """
    Проверка соединения с сервером лояльности KilBil
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled("KilBil"):
        return 2
    settings = requestSettings("KilBil")
    url = getValue(settings, "url")
    if not url:
        raise Exception("Не указан адрес сервера")
    if url[-1] != '/':
        url += '/'
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    apiKey = getValue(settings, "apiKey")
    try:
        resp = requests.get(url + "load/getprograms?h=" + apiKey, timeout=timeout)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testUds():
    """
    Проверка соединения с сервером лояльности UDS
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled("UDS"):
        return 2
    settings = requestSettings("UDS")
    url = getValue(settings, "serverAddress")
    if not url:
        raise Exception("Не указан адрес сервера")
    if url[-1] != '/':
        url += '/'
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    auth = HTTPBasicAuth(getValue(settings, "companyId"), getValue(settings, "apiToken"))
    try:
        resp = requests.get(url + "tags", timeout=timeout, auth=auth)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


def testLoymax():
    """
    Проверка соединения с сервером лояльности Loymax
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled(pluginName):
        return 2
    settings = requestSettings("Loymax")
    url = getValue(settings, "serverUrl")
    if not url:
        raise Exception("Не указан адрес сервера")
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    auth = HTTPBasicAuth(getValue(settings, "user"), getValue(settings, "password"))
    data = et.Element("XMLRequest")
    data.set("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
    data.set("xmlns:xsd", "http://www.w3.org/2001/XMLSchema")
    et.SubElement(data, "Version")
    try:
        resp = requests.post(url, data=et.tostring(data), timeout=timeout, auth=auth)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception as e:
        return 1


def testManzana(pluginName):
    """
    Проверка соединения с сервером лояльности Manzana и Manzana2019
    :return: 0 - связь есть, 1 - связи нет, 2 - плагин выключен
    """
    if not pluginEnabled(pluginName):
        return 2
    settings = requestSettings(pluginName)
    url = getValue(settings, "url")
    if not url:
        raise Exception("Не указан адрес сервера")
    timeout = getValue(settings, "timeout")
    timeout = 10 if not timeout else int(timeout)
    auth = HTTPBasicAuth(getValue(settings, "user"), getValue(settings, "password"))
    try:
        resp = requests.get(url, timeout=timeout, auth=auth)
        resp.close()
        return 0 if resp.status_code == 200 else 1
    except Exception:
        return 1


"""
Скипт возвращает одно из 4 значений:
0 - связть с сервисом есть
1 - связи с сервисом нет
2 - плагин на кассе выключен
3 - неизвестное название плагина
"""
pluginName = sys.argv[1].lower()
try:
    if pluginName == "artixbonus" or pluginName == "artixcertificate":
        print(testArtixBonus(pluginName))
    elif pluginName == "onlinecoupon":
        print(testArtixCoupon())
    elif pluginName == "onlinecards":
        print(testOnlineCards())
    elif pluginName == "discountcounters":
        print(testDiscountCounters())
    elif pluginName == "cardissuance":
        print(testCardIssuance())
    elif pluginName == "rsloyalty" or pluginName == "rsloyaltycertificate":
        print(testRsLoyalty(pluginName))
    elif pluginName == "kilbil":
        print(testKilBil())
    elif pluginName == "uds":
        print(testUds())
    elif pluginName == "loymax":
        print(testLoymax())
    elif pluginName == "manzana" or pluginName == "manzana2019":
        print(testManzana(pluginName))
    else:
        print(3)
except Exception as e:
    print(e)
